@@ -4,7 +4,7 @@ use Moose;
 
 extends 'Webservice::InterMine::Constraint';
 with 'Webservice::InterMine::Constraint::Role::Operator';
-use InterMine::TypeLibrary qw(BinaryOperator);
+use Webservice::InterMine::Types qw(BinaryOperator);
 use MooseX::Types::Moose qw(Str);
 
 has '+op' => ( isa => BinaryOperator, coerce => 1);
